@@ -1,9 +1,9 @@
 from azfs.az_file_client import (
-    AzFileClient
+    AzFileClient,
+    export_decorator
 )
 
 from azfs.az_file_system import AzFileSystem
-from azfs.export_decorator import ExportDecorator
 from azfs.utils import BlobPathDecoder
 
 from .table_storage import (
@@ -12,8 +12,15 @@ from .table_storage import (
 )
 
 # comparable tuple
-VERSION = (0, 2, 10)
+VERSION = (0, 2, 11)
 # generate __version__ via VERSION tuple
 __version__ = ".".join(map(str, VERSION))
 
-__all__ = ["AzFileClient", "AzFileSystem", "BlobPathDecoder", "TableStorage", "TableStorageWrapper", "ExportDecorator"]
+__all__ = [
+    "AzFileClient",
+    "AzFileSystem",
+    "BlobPathDecoder",
+    "TableStorage",
+    "TableStorageWrapper",
+    "export_decorator"
+]
