@@ -14,7 +14,12 @@ from azure.core.exceptions import ResourceNotFoundError
 from azfs.clients.blob_client import AzBlobClient
 from azfs.clients.datalake_client import AzDataLakeClient
 from azfs.clients.client_interface import ClientInterface
-from azfs.error import AzfsInputError
+from azfs.error import (
+    AzfsInputError,
+    AzfsDecoratorFileFormatError,
+    AzfsDecoratorReturnTypeError,
+    AzfsDecoratorSizeNotMatchedError
+)
 import pandas as pd
 
 
@@ -702,7 +707,7 @@ class TestExists:
 
 
 class TestExportDecorator:
-    decorator = azfs.ExportDecorator()
+    decorator = azfs.export_decorator
 
     @staticmethod
     @decorator.register()
@@ -736,6 +741,17 @@ class TestExportDecorator:
         """
         return pd.DataFrame(), pd.DataFrame
 
+    @staticmethod
+    @decorator.register()
+    def export_str_example_multiple(_input: str) -> (str, str):
+        """
+
+        Args:
+            _input: some_name
+
+        """
+        return "a", "b"
+
     azc = azfs.AzFileClient()
     azc.import_decorator(decorator, keyword_list=["prod"])
 
@@ -750,7 +766,7 @@ class TestExportDecorator:
     )
 
     def test_return_type_not_matched(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(AzfsDecoratorReturnTypeError):
             self.azc.export_df_example_1(
                 _input="error",
                 prod_file_name_prefix="prefix",
@@ -758,8 +774,18 @@ class TestExportDecorator:
                 prod_file_name_suffix="suffix"
             )
 
+        with pytest.raises(AzfsDecoratorReturnTypeError):
+            self.azc.export_df_example_1(
+                _input="error",
+                prod_output_parent_path="https://prodazfs.dfs.core.windows.net/test_caontainer",
+                prod_key="test",
+                prod_file_name_prefix="prefix",
+                prod_file_name="the_file_name",
+                prod_file_name_suffix="suffix"
+            )
+
     def test_format_type_not_matched(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(AzfsDecoratorFileFormatError):
             self.azc.export_df_example_2(
                 _input="error",
                 prod_output_parent_path="https://testazfs.dfs.core.windows.net/test_caontainer",
@@ -769,7 +795,7 @@ class TestExportDecorator:
                 prod_format_type="parquet"
             )
 
-        with pytest.raises(ValueError):
+        with pytest.raises(AzfsDecoratorFileFormatError):
             self.azc.export_df_example_3(
                 _input="error",
                 prod_storage_account="testazfs",
@@ -781,7 +807,7 @@ class TestExportDecorator:
                 prod_format_type="parquet"
             )
 
-        with pytest.raises(ValueError):
+        with pytest.raises(AzfsDecoratorFileFormatError):
             self.azc_multiple.export_df_example_multiple(
                 _input="error",
                 prod_file_name_prefix="prefix",
@@ -798,3 +824,35 @@ class TestExportDecorator:
                 dev_format_type="parquet",
             )
 
+        with pytest.raises(AzfsDecoratorSizeNotMatchedError):
+            self.azc_multiple.export_df_example_multiple(
+                _input="error",
+                prod_file_name_prefix="prefix",
+                prod_file_name="the_file_name_1",
+                prod_file_name_suffix="suffix",
+                prod_format_type="parquet",
+
+                dev_storage_account="devazfs",
+                dev_container="test_container",
+                dev_file_name_prefix="prefix",
+                dev_file_name=["the_file_name_1", "the_file_name_2"],
+                dev_file_name_suffix="suffix",
+                dev_format_type="parquet",
+            )
+
+        with pytest.raises(AzfsDecoratorReturnTypeError):
+            self.azc_multiple.export_str_example_multiple(
+                _input="error",
+                prod_file_name_prefix="prefix",
+                prod_file_name=["the_file_name_1", "the_file_name_2"],
+                prod_file_name_suffix="suffix",
+                prod_format_type="parquet",
+
+                dev_storage_account="devazfs",
+                dev_container="test_container",
+                dev_key="some_folder",
+                dev_file_name_prefix="prefix",
+                dev_file_name=["the_file_name_1", "the_file_name_2"],
+                dev_file_name_suffix="suffix",
+                dev_format_type="parquet",
+            )
